function [CxZ,SxZ,CxZd,SxZd,B1Wf] = csxz(Lz,Nsz,dz,Cx,Sx,Bzc0,M,Gz,gammabar,dt)
%
% Caulcate CxZ and SxZ
debug_on = 0;
z = -Lz/2:(Lz/Nsz):(Lz/2-Lz/Nsz);
z = z(:);
indzLb = find(z>-dz/2,1,'first');
indzRb = find(z<dz/2,1,'last');
Nx = length(Cx(:));

CxZd = ones(Nsz,Nx);
SxZd = zeros(Nsz,Nx);
CxZ = ones(Nsz,Nx);
SxZ = zeros(Nsz,Nx);
CxPhZ = zeros(Nsz,Nx);

[Axc,Bxc] = cs2ab(Cx,Sx);
[~,~,psi,theta] = ab2cs(Axc,Bxc);
if debug_on == 1
    Axc = reshape(Axc,1,[]);
    Bxc = reshape(Bxc,1,[]);
    AxcZ = ones(Nsz,Nx);
    BxcZ = zeros(Nsz,Nx);
    for indz = indzLb:indzRb
        AxcZ(indz,:) = Axc;
        BxcZ(indz,:) = Bxc;
    end
end

% Inverse iteration
phi = zeros(Nsz,1);
% Calculate Cx and Sx for each z
for indx = Nx:-1:1
    % The desired CxZ and SxZ that will reduce the order of A and B
    % polynomial order by 1
    for indz = indzLb:indzRb
        CxZd(indz,indx) = Cx(indx);
        SxZd(indz,indx) = Sx(indx)*exp(1i*2*phi(indz));
    end
    
    % Adjust the phase of SxZ based on a feasible phase of CxZ
    % The alternative phase is corresonding to the alternative z gradient
    [temp1,~,temp2] = bc2ab(-1i*exp(1i*theta(indx)).*(Bzc0.*sin(psi(indx)/2)),Nsz);
    CxPhZ(:,indx) = (-1)^indx.*temp2;
    CxZd(:,indx) = CxZd(:,indx).*exp(1i*CxPhZ(:,indx));
    SxZd(:,indx) = SxZd(:,indx).*exp(1i*CxPhZ(:,indx));
    
    % Design the RF subpulse that approximate the desired CxZd and SxZd
    [Bzc,Bt] = b2bc(temp1.*exp(1i*(2*phi(:)+CxPhZ(:,indx))),M);
    if mod(indx,2) == 0     % positive slice gradient
        Azc = b2a(Bzc);
    else                              % negative slice gradient
        Azc = conj(b2a(Bzc));
        Bzc = Bzc(end:-1:1);
    end
    B1Wf(:,indx) = ab2rf(Azc,Bzc,gammabar,dt);
    
    % Calculate CxZ and SxZ using Bloch equation simulation
    Gzt = Gz*(-1)^indx;
    aRep = exp(-1i*(2*pi*gammabar*Gzt*dt)*z*(M)/4);
    bRep = 0;
    a0 = ones(Nsz,1);
    b0 = zeros(Nsz,1);
    a1 = ones(Nsz,1);
    b1 = zeros(Nsz,1);
    % slr
    [aRf,bRf] = bloch(B1Wf(:,indx),Gzt,z(:),gammabar,dt);
    % cascade the rotation
    % rephase 1
    [a1,b1] = rab(aRep,bRep,a0,b0);
    % SLR
    [a1,b1] = rab(aRf,bRf,a1,b1);
    % rephase 2
    [a1,b1] = rab(aRep,bRep,a1,b1);
    CxZ(:,indx) = a1(:);
    SxZ(:,indx) = b1(:);
    
    if debug_on == 1
        % display the comparison
        temp = 1:Nsz;
        figure(701)
        clf(701);
        subplot(221); hold on;
        plot(real(Bt(:)),'k')
        plot(real(SxZd(temp,indx)),'b');
        plot(real(SxZ(temp,indx)),'rx');
        title('Real(Sx)');
        subplot(223); hold on;
        plot(imag(Bt(:)),'k')
        plot(imag(SxZd(temp,indx)),'b');
        plot(imag(SxZ(temp,indx)),'rx');
        title('Imag(Sx)');
        % Cx
        subplot(222); hold on;
        plot(real(CxZd(temp,indx)),'r');
        plot(real(CxZ(temp,indx)),'rx');
        title('Real(Cx)');
        subplot(224); hold on;
        plot(imag(CxZd(temp,indx)),'b');
        plot(imag(CxZ(temp,indx)),'bo');
        title('Imag(Cx)');
        set(gcf,'Name',['Indx = ',num2str(indx), ' : Compare SxZd CxZd with a and b']);
        pause
    end
    
    % update the phase
    phi = phi + CxPhZ(:,indx);
    
    if debug_on == 1
        if indx > 1
            % Reduce the order of A and B polynomial by 1
            tempA = zeros(size(AxcZ));
            tempB = zeros(size(BxcZ));
            for indz = indzLb:indzRb
                tempA(indz,:) = conj(CxZ(indz,indx))*AxcZ(indz,:) + conj(SxZ(indz,indx))*BxcZ(indz,:);
                tempB(indz,:) = -SxZ(indz,indx)*AxcZ(indz,:) + CxZ(indz,indx)*BxcZ(indz,:);
                AxcZ(indz,1:indx-1) = tempA(indz,1:indx-1);
                BxcZ(indz,1:indx-1) = tempB(indz,2:indx);
                AxcZ(indz,indx:end) = 0;
                BxcZ(indz,indx:end) = 0;
            end
            sum(tempA(indzLb:indzRb,end),1),
            sum(tempB(indzLb:indzRb,1),1),
        end
    end
    
    mc = 0;
end