function [ Fp, Fs ] = firfpfs( deltap,deltas,BW,N,phaseType)
% Calculate Fp and Fs for a linear phase  FIR filter
if nargin < 5
    phaseType = 'linear';
end

a1 = 5.309e-3;      a2 = 7.114e-2;      a3 = -4.761e-1;
a4 = -2.66e-3;      a5 = -5.941e-1;     a6 = -4.278e-1;

% Calculate Dinf
switch phaseType
    case {'linear'}
        tempL1 = log10(deltap);
        tempL2 = log10(deltas);
        Dinf = (a1*tempL1^2 + a2*tempL1+a3)*tempL2+(a4*tempL1^2+a5*tempL1+a6);
    case{'minphase','maxphase'}
        tempL1 = log10(2*deltap);
        tempL2 = log10(deltas^2/2);
        Dinf = (a1*tempL1^2 + a2*tempL1+a3)*tempL2+(a4*tempL1^2+a5*tempL1+a6);
        Dinf = Dinf/2;
    otherwise
        disp('By default, linear phase design');
        tempL1 = log10(deltap);
        tempL2 = log10(deltas);
        Dinf = (a1*tempL1^2 + a2*tempL1+a3)*tempL2+(a4*tempL1^2+a5*tempL1+a6);
end

% Calculate Fp and Fs
Fp = 1/2*(BW-Dinf/(N-1));                   % cutoff frequency of passband
Fs = 1/2*(BW+Dinf/(N-1));                   % cutoff frequency of stopband

end
