function [ Ac,A ] = firmat( N,w )
% Generate complex and real matrices for FIR filter design
% w: a vector defines a discrete set of sampling point on the frequency
% axis
% N: order of the FIR filter, an odd number
% Ac: a complex matrix with complex exponential functions as basis
% functions
% A: a real matrix with cosine and sin functions as basis functions
%

% Build a complex matrix
n = -(N-1)/2:(N-1)/2;
n = n(:);
[nn,ww] = meshgrid(n,w(:));
Ac = exp(-1i*ww.*nn);

% Build a real matrix
A = zeros(length(w),N);
% constant term
A(:,1) = 1;
% cosine terms
k = 1:(N-1)/2;
[kk,ww] = meshgrid(k,w);
temp = cos(kk.*ww);
A(:,2:(N-1)/2+1) = temp;
% sine terms
temp = sin(kk.*ww);
A(:,(N-1)/2+2:end) = temp;

end

