function [ b,r,H,output] = firmininf( d,N,Fp,Fs,Wp,Ws,deltap,deltas,opt)
% Design a minphase complex FIR filter using a min-max optimization process
% Input:
% d, the desired excitation pattern
% N, the order of the FIR filter
% Fp,Fs, the position of the passband and the stopband
% Wp, Ws, the weight of the passband and the stopband
% Output:
% b, the coefficients of the designed of FIR filter
% as basis functions
% output: additional parameters
%

% design a linear phase FIR filter of order 2*N-1
[ ~,r,~,output ] = firlininf( d.^2,2*N-1,Fp,Fs,Wp,Ws,opt);

% spectral factorization
M = 8*length(d);
w = 0:(2*pi/M):(2*pi-(2*pi/M));
[~,A] = firmat(2*N-1,w);
temp = A*r;
MagX = sqrt(temp + abs(min(temp)) + eps);
X = MagX.*exp(-1i*imag(hilbert(log(MagX))));
temp = ifft(X);
b = temp(1:N).*sqrt((2+2*deltap^2-deltas^2)/2);

% calculate the frequency response
M = length(d);
w = (-pi):(2*pi/M):(pi-(2*pi/M));
Ac = firmat(N,w);
H = Ac*b;
end

