function [k,s] = g2ks(g,dt)
% g2kx
% Converts gradient to k and slew rate.
% Note: only for excitation k-space definition
% Input:
%  g: Nx1 complex vector for gradient waveform (Gauss/cm)
%  dt: a scalar representing the time interval (sec) between two neighboring samples
% Output:
%  k: Nx1 complex vector for k-space samples (rad/cm)
%  s: Nx1 complex vector for slew rate waveform (Gauss/cm/sec)

gamma = 4257.6;  % Hz/Gauss
g = g(:);
k = 2*pi*gamma*(cumsum(g)-sum(g))*dt;  % (rad/cm)
s = [0; diff(g)/dt];  % Gauss/cm/sec