function genEPI2DGrad( g, NEchoCenterToEnd, dt, fname)
% generate a 2D EPI gradient waveform
% Chao
%

if nargin < 3
    dt = 10*1e-6;
end
if nargin < 4
    fname = 'ZPL_chao_epi2d_grad.g';
end

debug_on = 0;

% oversampling factor
Nos = round(dt/1e-5);

% generate the waveform 
gx = g(2,:)*10;                     % from Gauss/cm to mT/m
gy = g(1,:)*10;                     % from Gauss/cm to mT/m
gxMax = max(abs(gx));
gyMax = max(abs(gy));
if gxMax>1e-5
    gxNormalized = gx./gxMax;
else
    gxNormalized = 0*gx;
end
if gyMax>1e-5
    gyNormalized = gy./gyMax;
else
    gyNormalized = 0*gy;
end
gxNormalizedOs = reshape(repmat(gxNormalized,[Nos,1]),[],1);
gyNormalizedOs = reshape(repmat(gyNormalized,[Nos,1]),[],1);
Nt = length(gxNormalizedOs);

% write gradient waveform
fid = fopen(fname,'w');
fwrite(fid,Nt,'int');
fwrite(fid,NEchoCenterToEnd*Nos,'int');
fwrite(fid,gxMax,'float');
fwrite(fid,gyMax,'float');
fwrite(fid,gxNormalizedOs,'float');
fwrite(fid,gyNormalizedOs,'float');
fclose(fid);

if(debug_on)
    figure;
    plot(gxNormalizedOs);
    figure;
    plot(gyNormalizedOs);
end

end