function B1Wf = mdslr( mdslrpara )
% multidimensional RF pulse design
%

D = mdslrpara.D;        
Lx = mdslrpara.Lx;
Nx = mdslrpara.Nx;
dz = mdslrpara.dz;
Lz = mdslrpara.Lz;
Fpz = mdslrpara.Fpz;
Fsz = mdslrpara.Fsz;
Nz = mdslrpara.Nz;
gammabar = mdslrpara.gammabar;
dt = mdslrpara.dt;
phmod = 1;

% minimum phase design
if isfield(mdslrpara,'phaseType')
    phaseType = mdslrpara.phaseType;
else
    phaseType = 'linear';
end

% determine weighting
if isfield(mdslrpara,'delta')
    delta = mdslrpara.delta;
    deltapz = delta(1,1);
    deltasz = delta(1,2);
    deltapx = delta(2,1);
    deltasx = delta(2,2);
    % Wsx and Wpx
    [Wpx,Wsx] = delta2w(deltapx,deltasx,phaseType);
    % Wsz and Wpz
    [Wpz,Wsz] = delta2w(deltapz,deltasz);
else
    Wpz = 1;
    Wsz = 1;
    Wpx = 1;
    Wsx = 1;
end

Nsz = size(D,1);
Nsx = size(D,2);
x = (-Lx/2):(Lx/Nsx):(Lx/2-(Lx/Nsx));
z = (-Lz/2):(Lz/Nsz):(Lz/2-(Lz/Nsz));
indzLb = find(z>-dz/2,1,'first');
indzRb = find(z<dz/2,1,'last');

% For each z, calculate the desired rotation, magnitude
SxZd = zeros(Nsz,Nx);
CxZd = ones(Nsz,Nx);
CxPhZ = zeros(Nsz,Nx);
PsixZd = zeros(Nsz,Nx);
ThetaxZd = zeros(Nsz,Nx);
% initialize FpxR, FpxL, FsxL, FsxR
if isfield(mdslrpara,'Fpx')
    if size(mdslrpara.Fpx,1) == 1
        Fpx = mdslrpara.Fpx.*ones(Nsz,1);
        Fsx = mdslrpara.Fsx.*ones(Nsz,1);
    else
        Fpx = mdslrpara.Fpx;
        Fsx = mdslrpara.Fsx;
    end
    FpxR = Fpx;
    FpxL = -Fpx;
    FsxR = Fsx;
    FsxL = -Fsx;
elseif isfield(mdslrpara,'FpxR')
    if size(mdslrpara.FpxR,1) == 1
        FpxR = mdslrpara.FpxR.*ones(Nsz,1);
        FpxL = mdslrpara.FpxL.*ones(Nsz,1);
        FsxR = mdslrpara.FsxR.*ones(Nsz,1);
        FsxL = mdslrpara.FsxL.*ones(Nsz,1);
    else
        FpxR = mdslrpara.FpxR;
        FpxL = mdslrpara.FpxL;
        FsxR = mdslrpara.FsxR;
        FsxL = mdslrpara.FsxL;
    end
end
% determine the desired rotation
opt = [];
opt.design = 'real';
opt.quiet = 1;
for indz = indzLb:indzRb
    if max(abs(FpxR(indz)),abs(FpxL(indz))) > 0.05
        d = D(indz,:);
        if strcmp(phaseType,'linear')
            [Bxc,~,p,output] = firlininf(d(:),Nx,[FpxL(indz),FpxR(indz)],...
                [FsxL(indz),FsxR(indz)],Wpx,Wsx,opt);
        elseif strcmp(phaseType,'minphase')
            [ Bxc,~,p,output ] = firmininf(d(:)./max(abs(d(:))),Nx,[FpxL(indz),FpxR(indz)],...
                [FsxL(indz),FsxR(indz)],Wpx,Wsx,deltapx,deltasx,opt);
            Bxc = Bxc*max(abs(d(:)));
            p = p.*max(abs(d(:)));
        elseif strcmp(phaseType,'maxphase')
            [ Bxc,~,p,output ] = firmininf(d(:)./max(abs(d(:))),Nx,[FpxL(indz),FpxR(indz)],...
                [FsxL(indz),FsxR(indz)],Wpx,Wsx,deltapx,deltasx,opt);
            Bxc = Bxc(end:-1:1)*max(abs(d(:)));
            p = p.*max(abs(d(:)));
        end
        Axc = b2a(Bxc);
        [Cx,Sx,psi,theta] = ab2cs(Axc,Bxc);

        CxZd(indz,:) = reshape(Cx,1,Nx);
        SxZd(indz,:) = reshape(Sx,1,Nx);
        PsixZd(indz,:) = reshape(psi,1,Nx);
        ThetaxZd(indz,:) = reshape(theta,1,Nx);      
    end
end

% Modify the phase of the desired rotation
if phmod == 1
    phi = zeros(Nsz,1);
    for indx = Nx:-1:1
        % The desired CxZ and SxZ that will reduce the order of A and B
        % polynomial order by 1
        SxZd(:,indx) = SxZd(:,indx).*exp(1i*2*phi);
        
        % Adjust the phase of SxZ based on a feasible phase of CxZ
        % The alternative phase is corresonding to the alternative z gradient
        CxPhZ(:,indx) = (-1)^indx.*(-imag(hilbert(log(sqrt(1-abs(SxZd(:,indx)).^2)))));
        CxZd(:,indx) = CxZd(:,indx).*exp(1i*(CxPhZ(:,indx)));
        SxZd(:,indx) = SxZd(:,indx).*exp(1i*(CxPhZ(:,indx)));
        
        % update phi
        phi = phi + CxPhZ(:,indx);
    end
end

% Design the RF subpulse that approximate the desired excitation pattern
opt = [];
opt.design = 'complex';
opt.quiet = 1;
B1Wf = zeros(Nz,Nx);
SxZ = zeros(Nsz,Nx);
CxZ = ones(Nsz,Nx);
% initialize FpzR, FpzL, FszL, FszR
if isfield(mdslrpara,'Fpz')
    Fpz = mdslrpara.Fpz;
    Fsz = mdslrpara.Fsz;
    FpzR = Fpz;
    FpzL = -Fpz;
    FszR = Fsz;
    FszL = -Fsz;
end
for indx = Nx:-1:1
    % Design a complex FIR filter
    d = SxZd(:,indx);
    [Bzc,~,~,output] = firlininf(d(:),Nz,[FpzL,FpzR],[FszL,FszR],Wpz,Wsz,opt);
    
    % Calculate the RF subpulse
    if mod(indx,2) == 0     % positive slice gradient
        Azc = b2a(Bzc);
    else                              % negative slice gradient
        Azc = conj(b2a(Bzc));
        Bzc = Bzc(end:-1:1);
    end
    B1Wf(:,indx) = ab2rf(Azc(:),Bzc(:),gammabar,dt);

end

end

