function [Ac,Bc,B1Wf] = rfslr(slrpara)
% calculate the A and B polynomial using SLR method
% Chao Ma
debug_on = 0;

% parameter preparation
BW = slrpara.BW;
deltape = slrpara.deltape;
deltase = slrpara.deltase;
fa = slrpara.fa;
N = slrpara.N;
if isfield(slrpara,'phaseType')
    phaseType = slrpara.phaseType;
else
    phaseType = 'linear';
end
if isfield(slrpara,'pulseType')
    pulseType = slrpara.pulseType;
else
    pulseType = 'ex';
end
if isfield(slrpara,'method')
    method = slrpara.method;
else
    method = 'continuous';
end

% Calculate deltap and deltas
[deltap,deltas] = erp2rp1d(deltape,deltase,pulseType);

% Calculate Dinf, Fp, Fs
[Fp, Fs] = firfpfs(deltap,deltas,BW,N,phaseType);

% get coefficients of Bn(z)
if strcmp(method,'continuous')
    if strcmp(phaseType,'linear')
        % Note both the amplitude and the sampling frequency are normailzed to 1
        [n,Fn,Ae,we] = firpmord([Fp Fs],[1 0],[deltap deltas], 1);
        % adjust pulse time if n > Nt;
        if(n>N)
            'increase pulse time';
            return;
        end
        n = N-1;
        Bc = sin(fa/2).*firgr(n,Fn,Ae,we);
    elseif strcmp(phaseType,'minphase') || strcmp(phaseType,'min')
        %     Bc = sin(fa/2).*firgr(n,Fn,Ae,we,phaseType);
        Bc = sin(fa/2).*firceqrip(N-1,BW,[deltap,deltas],'min');
    end
elseif strcmp(method,'discrete')
    d = slrpara.d;
    [Wp,Ws] = delta2w(deltap,deltas,phaseType);
    opt = [];
    if isreal(d)
        opt.design = 'real';
    else
        opt.design = 'complex';
    end
    opt.quiet = 1;
    if strcmp(phaseType,'linear')
            [Bc,~,~,output] = firlininf(d(:),N,Fp,Fs,Wp,Ws,opt);
        elseif strcmp(phaseType,'minphase') || strcmp(phaseType,'min')
            [Bc,~,~,output] = firmininf(d(:)./max(abs(d(:))),N,Fp,Fs,Wp,Ws,deltap,deltas,opt);
            Bc = Bc*max(abs(d(:)));
        end
end

% test the designed B polynomial
if debug_on == 1
    n = length(Bc);
    
    % plot the magnitude
    Bf = fftshift(fft(Bc,1024));
    w = linspace(-1,1,1024);
%     Bfd = zeros(size(w));
%     Bfd(abs(w)<BW/Fsam) = sin(fa/2);
    figure;
    plot(w,abs(Bf)); hold on;
%     plot(w,Bfd,'--r');
    
    % study the phaseType
    M = (n-1)/2;
    dw = 2*pi/length(Bf);
    w = -pi:dw:pi-dw;
    Bf_corr = exp(1i*M*w).*Bf;
    figure;
    plot(w/pi,real(Bf_corr)); hold on;
    plot(w/pi,imag(Bf_corr),'r');
    
end

% design polynomial A
Ac = b2a(Bc);

% design RF pulse
if nargout >2
    gammabar = slrpara.gammabar;
    dt = slrpara.dt;
    B1Wf = ab2rf(Ac, Bc, gammabar, dt);
end