function [ g,rf,t ] = sub_grf(grfpara)
% concentrate RF pulses, 
% gz without the hypothetical gradient lobe
% gz ramp does not merge with the gx blip
%

%% get parameters from grfpara
gxSub = grfpara.gxSub;
gzSub = grfpara.gzSub;
gzSubPlateauInd = grfpara.gzSubPlateauInd;
gzSubRe = grfpara.gzSubRe;
B1Wf = grfpara.B1Wf;

Nx = size(B1Wf,2);
NgxSub = size(gxSub,1);
NgzSub = size(gzSub,1);
NgzSubRe = length(gzSubRe);
if(size(gzSub,2) == 1)
    gzSub = repmat(gzSub,1,Nx);
end
if(size(gzSubPlateauInd,2) == 1)
    gzSubPlateauInd = repmat(gzSubPlateauInd,1,Nx);
end

if isfield(grfpara,'dt')
    dtG  = grfpara.dt;
elseif isfield(grfpara,'dtG')
    dtG = grfpara.dtG;
else
    dtG = 0;
end

if isfield(grfpara,'dtRf')
    dtRf = grfpara.dtRf;
else
    dtRf = dtG;
end

if isfield(grfpara,'dtSub')
    dtSub = grfpara.dtSub;
else
    dtSub = ones(size(gzSub))*dtG;
end

% oversampling RF pulse
if dtG == 0
    Nos = 1;
else
    Nos = round(dtRf/dtG);
end
B1WfOs = reshape(repmat(reshape(B1Wf,1,[]),[Nos,1]),[],Nx);

%% concentrate the RF pulse
gx = [];
gz = [];
rf = [];
t = [];
for indx = 1:Nx
    % add gx blip
    if indx > 1
        gx = [gx;gxSub(2:NgxSub-1)];
        gz = [gz;zeros(NgxSub-2,1)];
        rf = [rf;zeros(NgxSub-2,1);];
        t = [t;ones(NgxSub-2,1)*dtG];
    end
    
    % add gz gradient
    if mod(indx,2) == 1 % negative mainlobe
        gz = [gz;-gzSub(:,indx)];        
    else                          % positive mainlobe
        gz = [gz;gzSub(:,indx)];
    end
    gx = [gx;zeros(NgzSub,1)];
    t = [t;dtSub(:,indx)];
    rfSubStart = gzSubPlateauInd(1,indx);
    rfSubEnd = gzSubPlateauInd(2,indx);
    temp = zeros(NgzSub,1);
    temp(rfSubStart:rfSubEnd,1) = B1WfOs(:,indx);
    rf = [rf;temp(:)];
end
% add Gz rephase gradient
if isfield(grfpara,'gzReOn')
    gzReOn = grfpara.gzReOn;
else
    gzReOn = 1;
end
if (gzReOn == 1) && (isempty(gzSubRe) ~= 1)
    gz = [gzSubRe(1:end-1);gz(1:end-1);gzSubRe];
    gx = [zeros(NgzSubRe-1,1);gx;zeros(NgzSubRe-1,1)];
    rf = [zeros(NgzSubRe-1,1);rf;zeros(NgzSubRe-1,1)];
    t = [ones(NgzSubRe-1,1)*dtG;t;ones(NgzSubRe-1,1)*dtG];
elseif (gzReOn == 2) && (isempty(gzSubRe) ~= 1)
    gz = [gz(1:end-1);gzSubRe];
    gx = [gx;zeros(NgzSubRe-1,1)];
    rf = [rf;zeros(NgzSubRe-1,1)];
    t = [ones(NgzSubRe-1,1)*dtG;t;ones(NgzSubRe-1,1)*dtG];
end

% add rephase gradient of Gx
if isfield(grfpara,'gxSubRe')
    gxSubRe = grfpara.gxSubRe;
else
    gxSubRe = [];
end
if ~isempty(gxSubRe)
    gx = [gx;gxSubRe(2:end)];
    gz = [gz;zeros(size(gxSubRe(2:end)))];
    rf = [rf;zeros(size(gxSubRe(2:end)))];
    t = [t;ones(size(gxSubRe(2:end)))*dtG];
end
g = [gx(:),gz(:)];
g = g.';

end

