function [ g,rf,t ] = sub_grf_spsp(grfpara)
% concentrate RF pulses and gradients for a spsp pulse
%

%% get parameters from grfpara
gxAmp = grfpara.gxAmp;
gzSub = grfpara.gzSub;
gzSubPlateauInd = grfpara.gzSubPlateauInd;
gzSubRe = grfpara.gzSubRe;
B1Wf = grfpara.B1Wf;
Nx = size(B1Wf,2);
NgzSub = size(gzSub,1);
NgzSubRe = length(gzSubRe);

if(size(gzSub,2) == 1)
    gzSub = repmat(gzSub,1,Nx);
    gzSubRe = repmat(gzSubRe,1,Nx);
end
if(size(gzSubPlateauInd,2) == 1)
    gzSubPlateauInd = repmat(gzSubPlateauInd,1,Nx);
end

if isfield(grfpara,'dtG')
    dtG  = grfpara.dtG;
else
    dtG = 0;
end

if isfield(grfpara,'dtRf')
    dtRf  = grfpara.dtRf;
else
    dtRf = 0;
end
if isfield(grfpara,'gzSubRe2')
    gzSubRe2 = grfpara.gzSubRe2;
    NgzSubRe2 = length(gzSubRe2);
else
    gzSubRe2 = [];
    NgzSubRe2 = 0;
end
% add Gz rephase gradient
if isfield(grfpara,'gzReOn')
    gzReOn = grfpara.gzReOn;
else
    gzReOn = 0;
end

% oversampling RF pulse
if dtG == 0
    Nos = 1;
else
    Nos = round(dtRf/dtG);
end
B1WfOs = reshape(repmat(reshape(B1Wf,1,[]),[Nos,1]),[],Nx);

%% concentrate the RF pulse
gx = [];
gz = [];
rf = [];
for indx = 1:Nx
   % add gz main lobe
    gz = [gz;gzSub(1:end-1,indx);];
    rfSubStart = gzSubPlateauInd(1,indx);
    rfSubEnd = gzSubPlateauInd(2,indx);
    temp = zeros(NgzSub - 1,1);
    temp(rfSubStart:rfSubEnd,1) = B1WfOs(:,indx);
    rf = [rf;temp(:)];
    
    % add gz refocusing lobe
    if indx < Nx
        gz = [gz;gzSubRe(1:end-1,indx)];
        rf = [rf;zeros(NgzSubRe-1,1)];
    end
end

if (gzReOn == 1) && (isempty(gzSubRe2) ~= 1)
    gz = [gzSubRe2(1:end-1);gz;gzSubRe2];
    rf = [zeros(NgzSubRe2-1,1);rf;zeros(NgzSubRe2,1)];
elseif (gzReOn == 2) && (isempty(gzSubRe2) ~= 1)
    gz = [gz;gzSubRe2];
    rf = [rf;zeros(NgzSubRe2,1)];
else
    gz = [gz;0];
    rf = [rf;0];
end

% add rephase gradient of Gx
gx = ones(size(gz))*gxAmp;
g = [gx(:),gz(:)];
g = g.';

end

